
local sti = require "sti"
Camera = require "camera"
local bump = require 'bump'
local world = bump.newWorld()
local mouse = {borderImg = love.graphics.newImage("border.png")}
world.windowWidth = love.graphics.getWidth()
world.windowHeight = love.graphics.getHeight()
world.desktopWidth, world.desktopHeight = love.window.getDesktopDimensions()

local push = require "push"
world.gameWidth, world.gameHeight = 1366, 768 --fixed game resolution
push:setupScreen(world.gameWidth, world.gameHeight, world.desktopWidth, world.desktopHeight, {fullscreen = true})

local anim8 = require 'anim8'
local image, animation

local player = {pos={}}
player.pos.x = 1300
player.pos.y = 1800
player.img = love.graphics.newImage("player.png")
player.height = 40
player.width = 22

player.yspeed = 0
player.xspeed = 0

world.gravity = 9.8

testblock = {x=0, y=0, h=50, w=500}

                         -- frame, image,    offsets, border
  local rightRun = anim8.newGrid(445,600, 1841,2400,   0,600,     0)
  local leftRun = anim8.newGrid(445,600, 1841,2400,   0,1200,     0)

function love.load()
  
  
  image = love.graphics.newImage('SpriteSheet.png')

  animation = anim8.newAnimation(rightRun('1-4',1), 0.1)
  
  
  cam = Camera(player.pos.x, player.pos.y)
  -- Grab window size
  windowWidth  = love.graphics.getWidth()
  windowHeight = love.graphics.getHeight()
  -- Load map file
  map = sti.new("maps/map1.lua")
  
  world:add(player, player.pos.x, player.pos.y, player.width, player.height) -- x,y,w,h
  world:add(testblock, testblock.x, testblock.y, testblock.w, testblock.h) -- x,y,w,h
  
  --print(map.layers[1].data[1][1].gid)
  print(map.tileInstances[1][1].gid)
  
  for mapY = 1,100 do
    for mapX = 1,100 do
      if map.layers[1].data[mapY][mapX].gid ~= 9 then
        local block = {x=mapX*24-24, y=mapY*24-24}
        world:add(block, mapX*24-24, mapY*24-24, 24, 24)
      end
    end
  end
  
end

--#############FUNCTIONS##################
function love.mousepressed(x, y, button, istouch)
   if button == 1 then -- the primary button
      mouse.clickx = x+cam.x-world.desktopWidth/2
      mouse.clicky = y+cam.y-world.desktopHeight/2
      mouse.button = button
      local tilex = math.floor((x+cam.x-world.desktopWidth/2)/24)+1
      local tiley = math.floor((y+cam.y-world.desktopHeight/2)/24)+1
      mouse.clickedTile = map.layers[1].data[tiley][tilex].gid
      map.layers[1].data[tiley][tilex].gid = 9
   end
end

function love.update(dt)
  if love.keyboard.isDown("up") or love.keyboard.isDown("w") then
    player.pos.y = player.pos.y - player.yspeed
    player.yspeed = player.yspeed + world.gravity*dt
  else
    player.pos.y = player.pos.y - player.yspeed
    player.yspeed = player.yspeed - world.gravity*dt
  end
  
  function love.keypressed(key)
    if key == "left" or key == "a" then
      animation = anim8.newAnimation(leftRun('1-4',1), 0.1)
    elseif key == "right" or key == "d" then
      animation = anim8.newAnimation(rightRun('1-4',1), 0.1)
    end
    if key == 'escape' then
      love.event.quit()
    end
  end
  
  if love.keyboard.isDown("left") or love.keyboard.isDown("a") then
    player.pos.x = player.pos.x - player.xspeed
    player.xspeed = player.xspeed + world.gravity*dt
    animation:update(dt)
  elseif love.keyboard.isDown("right") or love.keyboard.isDown("d") then
    player.pos.x = player.pos.x - player.xspeed
    player.xspeed = player.xspeed - world.gravity*dt
    animation:update(dt)
  else
    
    if player.xspeed >= -1 and player.xspeed <= 1 then
      player.xspeed = 0
    elseif player.xspeed < -1 then
      player.pos.x = player.pos.x - player.xspeed
      player.xspeed = player.xspeed + world.gravity*dt
    elseif player.xspeed > -1 then
      player.pos.x = player.pos.x - player.xspeed
      player.xspeed = player.xspeed - world.gravity*dt
    end
    
  end
  
  local actualX, actualY, col, len = world:move(player, player.pos.x, player.pos.y)
  player.collision = col
  player.length = len
  player.pos.x, player.pos.y = math.floor(actualX), math.floor(actualY)
  
  if player.length >= 1 and (player.collision[1].normal.y == -1 or player.collision[1].normal.y == 1) then
    player.yspeed = 0
  end
  if player.length >= 1 and (player.collision[1].normal.x == -1 or player.collision[1].normal.x == 1) then
    player.xspeed = 0
  end
  
  local dx,dy = player.pos.x - cam.x, player.pos.y - cam.y
  function rubber_band(dx,dy)
    local dt = love.timer.getDelta()*10
    return dx*dt, dy*dt
  end
  dx,dy = rubber_band(dx,dy)
  cam:move(math.floor(dx/2), math.floor(dy/2))
  -- Update world
  --cam:lock(player.pos.x, player.pos.y)
 
  map:update(dt)
  mouse.x, mouse.y = love.mouse.getPosition() -- get the position of the mouse
  mouse.relX = mouse.x+cam.x-world.desktopWidth/2-(mouse.x+cam.x-world.desktopWidth/2)%24
  mouse.relY = mouse.y+cam.y-world.desktopHeight/2-(mouse.y+cam.y-world.desktopHeight/2)%24
end

function love.draw()
  love.graphics.setBackgroundColor(0, 0, 0, 255)
  push:apply("start")
  cam:attach()
  
  
  map:draw()
  love.graphics.rectangle("fill",testblock.x, testblock.y, testblock.w, testblock.h)
  --love.graphics.draw(player.img, player.pos.x, player.pos.y)
  love.graphics.draw(mouse.borderImg, mouse.relX, mouse.relY)
  love.graphics.setColor(0,0,0)
  love.graphics.print("X: "..tostring(mouse.clickx).." Y:"..tostring(mouse.clicky).." gid:"..tostring(mouse.clickedTile), player.pos.x, player.pos.y-30)
  love.graphics.setColor(255,255,255)
  animation:draw(image, player.pos.x, player.pos.y, 0, 0.08, 0.08, 75, 35)
  
  cam:detach()
  push:apply("end")
end